import bpy
from bpy.types import AddonPreferences
from bpy.props import BoolProperty
from bpy_extras import keyconfig_utils
from bl_keymap_utils import keymap_hierarchy
import rna_keymap_ui
from . import bl_info, operators, matcaps

class ShadingPlusPreferences(AddonPreferences):
    bl_idname = __package__
    
    show_preset_names: BoolProperty(
        name="Show Preset Names",
        default=False,
        description="Show names next to the preset icons in the pie menu"
    )
    
    use_eevee_next_optimized_settings: BoolProperty(
        name="Use EEVEE Next optimized settings when possible",
        default=True,
        description="Apply optimized EEVEE Next settings when using Blender 4.2 or above"
    )

    def draw(self, context):
        layout = self.layout  # Get layout from self
        
        # EEVEE Settings Section
        layout.separator()
        layout.label(text="EEVEE Settings")
        layout.prop(self, "use_eevee_next_optimized_settings")
        
        layout.separator()
        layout.label(text="Matcap Settings")
        row = layout.row()
        row.operator("shading_plus.install_matcaps", text="Update Matcaps", icon='FILE_REFRESH')
        
        # Draw Keymap Settings directly under Preferences
        self.draw_keymaps(context, layout)

    def draw_keymaps(self, context, layout):
        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user
        
        keymaps_info = [
            {
                'km_name': '3D View',
                'space_type': 'VIEW_3D',
                'region_type': 'WINDOW',
                'operator_idname': 'wm.call_menu_pie',
                'properties': {'name': 'PIE_MT_1P2D_shading_plus'},
                'label': 'Keymap',
            },
        ]
        
        for km_info in keymaps_info:
            km = kc.keymaps.get(km_info['km_name'])
            if not km:
                continue
                
            for kmi in km.keymap_items:
                if kmi.idname == km_info['operator_idname']:
                    layout.label(text=km_info['label'])
                    rna_keymap_ui.draw_kmi([], kc, km, kmi, layout, 0)
                    break




# REGISTER/UNREGISTER
classes = (
    ShadingPlusPreferences,
)

def register_keymap():
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')

        # Add the keymap for the operator
        kmi = km.keymap_items.new('PIE_MT_1P2D_shading_plus', 'Z', 'PRESS')
        kmi.active = True

def unregister_keymap():
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = kc.keymaps.get('3D View')
        if km:
            for kmi in km.keymap_items:
                if kmi.idname == 'PIE_MT_1P2D_shading_plus':
                    km.keymap_items.remove(kmi)
                    break


def register():
    for cls in classes:
        try:
            bpy.utils.unregister_class(cls)
        except RuntimeError:
            pass
        bpy.utils.register_class(cls)

    register_keymap()

def unregister():
    unregister_keymap()

    for cls in classes:
        bpy.utils.unregister_class(cls)


